# coding=utf-8
#
# Copyright (C) 2014 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

from __future__ import print_function, division, absolute_import
import os.path
from mako.template import Template
from textwrap import dedent

from modules import utils


def gen_header(status):
    """
    Generate a GLSL program header.

    Generate header code for ARB_shader_image_load_store GLSL parser
    tests that are expected to give status as result.
    """
    return dedent("""\
        /*
         * [config]
         * expect_result: {0}
         * glsl_version: 1.50
         * require_extensions: GL_ARB_shader_image_load_store
         * [end config]
         */
        #version 150
        #extension GL_ARB_shader_image_load_store: require
    """.format(status))


def gen_vector_type(scalar, dim):
    """
    Generate a GLSL vector type name.

    Generate a GLSL vector type based on the given scalar type with the
    specified number of dimensions.
    """
    vector_types = {
        'int': 'ivec',
        'uint': 'uvec',
        'float': 'vec',
    }

    return ('{0}{1}'.format(vector_types[scalar], dim) if dim > 1
            else scalar)


def gen_scalar_args(arity):
    """
    Generate image builtin scalar arguments.

    Returns a generator of well-formed scalar arguments for an image
    built-in of the given arity having t as argument type.
    """
    return lambda t: ', {0}(0)'.format(t) * arity


def gen_vector_args(arity):
    """
    Generate image builtin vector arguments.

    Returns a generator of well-formed arguments for an image built-in
    of the given arity having a 4-component vector of t as argument
    type.
    """
    return lambda t: ', {0}(0)'.format(gen_vector_type(t, 4)) * arity


def gen_address_args(dim):
    """
    Generate image address arguments.

    Returns a generator for the coordinate argument of an image
    built-in expecting an address of the given dimensionality.  If
    fail is True the generated argument will have an arbitrary
    incorrect dimensionality.
    """
    return (lambda fail = False: ', {0}(0)'.format(
        gen_vector_type('int', dim if not fail else dim % 3 + 1)))


def gen_address_args_ms(dim):
    """Generate multisample image address arguments."""
    return (lambda fail = False:
            gen_address_args(dim)() + (', 0' if not fail else ''))


def product(ps, *qss):
    """
    Generate the cartesian product of a number of lists of dictionaries.

    Each generated element will be the union of some combination of
    elements from the iterable arguments.  The resulting value of each
    'name' item will be the concatenation of names of the respective
    element combination separated with dashes.
    """
    for q in (product(*qss) if qss else [{}]):
        for p in ps:
            r = dict(p, **q)
            r['name'] = '-'.join(s['name'] for s in (p, q) if s.get('name'))
            yield r


def gen(name, src, tests):
    """
    Expand a source template for the provided list of test definitions.

    Generate a GLSL parser test for each of the elements of the
    'tests' iterable, each of them should be a dictionary of
    definitions that will be used as environment to render the source
    template.

    The file name of each test will be the concatenation of the 'name'
    argument with the 'name' item from the respective test dictionary.
    """
    template = Template(dedent(src))

    for t in product([{'name': name}], tests):
        filename = os.path.join('spec',
                                'ARB_shader_image_load_store',
                                'compiler',
                                '{0}.{1}'.format(t['name'],
                                                 t['shader_stage']))
        print(filename)

        dirname = os.path.dirname(filename)
        utils.safe_makedirs(dirname)

        with open(filename, 'w') as f:
            f.write(template.render(header = gen_header, **t))


shader_stages = [
    {'shader_stage': 'frag'},
    {'shader_stage': 'vert'}
]


image_atomic_builtins = [
    {
        'name': 'atomic-add',
        'image_builtin': 'imageAtomicAdd',
        'image_args': gen_scalar_args(1)
    },
    {
        'name': 'atomic-min',
        'image_builtin': 'imageAtomicMin',
        'image_args': gen_scalar_args(1)
    },
    {
        'name': 'atomic-max',
        'image_builtin': 'imageAtomicMax',
        'image_args': gen_scalar_args(1)
    },
    {
        'name': 'atomic-and',
        'image_builtin': 'imageAtomicAnd',
        'image_args': gen_scalar_args(1)
    },
    {
        'name': 'atomic-or',
        'image_builtin': 'imageAtomicOr',
        'image_args': gen_scalar_args(1)
    },
    {
        'name': 'atomic-xor',
        'image_builtin': 'imageAtomicXor',
        'image_args': gen_scalar_args(1)
    },
    {
        'name': 'atomic-exchange',
        'image_builtin': 'imageAtomicExchange',
        'image_args': gen_scalar_args(1)
    },
    {
        'name': 'atomic-comp-swap',
        'image_builtin': 'imageAtomicCompSwap',
        'image_args': gen_scalar_args(2)
    }
]


image_load_builtin = [
    {
        'name': 'load',
        'image_builtin': 'imageLoad',
        'image_args': gen_vector_args(0)
    }
]


image_store_builtin = [
    {
        'name': 'store',
        'image_builtin': 'imageStore',
        'image_args': gen_vector_args(1)
    }
]


image_types = [
    {
        'name': '1d',
        'image_type': 'image1D',
        'image_addr': gen_address_args(1)
    },
    {
        'name': '2d',
        'image_type': 'image2D',
        'image_addr': gen_address_args(2)
    },
    {
        'name': '3d',
        'image_type': 'image3D',
        'image_addr': gen_address_args(3)
    },
    {
        'name': '2d-rect',
        'image_type': 'image2DRect',
        'image_addr': gen_address_args(2)
    },
    {
        'name': 'cube',
        'image_type': 'imageCube',
        'image_addr': gen_address_args(3)
    },
    {
        'name': 'buffer',
        'image_type': 'imageBuffer',
        'image_addr': gen_address_args(1)
    },
    {
        'name': '1d-array',
        'image_type': 'image1DArray',
        'image_addr': gen_address_args(2)
    },
    {
        'name': '2d-array',
        'image_type': 'image2DArray',
        'image_addr': gen_address_args(3)
    },
    {
        'name': 'cube-array',
        'image_type': 'imageCubeArray',
        'image_addr': gen_address_args(3)
    },
    {
        'name': '2d-ms',
        'image_type': 'image2DMS',
        'image_addr': gen_address_args_ms(2)
    },
    {
        'name': '2d-ms-array',
        'image_type': 'image2DMSArray',
        'image_addr': gen_address_args_ms(3)
    }
]


def main():
    """Main function."""
    #
    # Test the early_fragment_tests layout qualifier.
    #
    gen('early-fragment-tests', """\
        ${header('pass' if shader_stage == 'frag' and hunk == 'in' else 'fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "Fragment shaders also allow the following layout qualifier on 'in'
         *  only (not with variable declarations):
         *
         *  layout-qualifier-id
         *    early_fragment_tests"
         */
        layout(early_fragment_tests) ${hunk};

        void main()
        {
        }
    """, product(shader_stages, [
        {'name': 'in', 'hunk': 'in'},
        {'name': 'out', 'hunk': 'out'},
        {'name': 'uniform', 'hunk': 'uniform uint u'},
        {'name': 'in-var', 'hunk': 'in float f'},
        {'name': 'uniform-buffer', 'hunk': 'uniform b { image2D img; }'}
    ]))

    #
    # Test image declarations.
    #
    gen('declaration-allowed', """\
        ${header('pass')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "[Images] can only be declared as function parameters or uniform
         *  variables."
         */
        layout(rgba32f) uniform ${image_type} img;
        layout(rgba32ui) uniform u${image_type} uimg;
        layout(rgba32i) uniform i${image_type} iimg;

        void f(${image_type} arg,
               u${image_type} uarg,
               i${image_type} iarg)
        {
        }

        void main()
        {
        }
    """, product(image_types, shader_stages))

    gen('declaration-global', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "[Images] can only be declared as function parameters or uniform
         *  variables."
         */
        layout(rgba32f) ${qualifier} image2D img;

        void main()
        {
        }
    """, product(shader_stages, [
        {'name': 'const', 'qualifier': 'const'},
        {'name': 'in', 'qualifier': 'in'},
        {'name': 'out', 'qualifier': 'out'}
    ]))

    gen('declaration-local', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "[Images] can only be declared as function parameters or uniform
         *  variables."
         */
        void main()
        {
            layout(rgba32f) image2D img;
        }
    """, shader_stages)

    gen('declaration-uniform-block', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "Sets of uniforms, except for samplers and images, can be grouped
         *  into uniform blocks."
         */
        uniform b {
            layout(rgba32f) image2D img;
        };

        void main()
        {
        }
    """, shader_stages)

    gen('declaration-argument', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "Images cannot be treated as l-values; hence, they cannot be used
         *  as out or inout function parameters, nor can they be assigned
         *  into."
         */
        void f(${qualifier} image2D y)
        {
        }

        void main()
        {
        }
    """, product(shader_stages, [
        {'name': 'inout', 'qualifier': 'inout'},
        {'name': 'out', 'qualifier': 'out'}
    ]))

    gen('declaration-initializer', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "[Images] cannot be declared with an initializer in a shader."
         */
        layout(rgba32f) uniform image2D img = 0;

        void main()
        {
        }
    """, shader_stages)

    gen('declaration-format-qualifier', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "It is an error to declare an image variable where the format
         *  qualifier does not match the image variable type."
         */
        layout(${format}) uniform ${prefix}${image_type} img;

        void main()
        {
        }
    """, product(image_types, shader_stages, [
        {'name': 'float',
         'format': 'rgba32f',
         'prefix': 'i'},
        {'name': 'int',
         'format': 'rgba32i',
         'prefix': 'u'},
        {'name': 'uint',
         'format': 'rgba32ui',
         'prefix': ''}
    ]))

    gen('declaration-format-qualifier-duplicate', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "Only one format qualifier may be specified for any image variable
         *  declaration."
         */
        layout(rgba32f) layout(rgba32f) uniform image2D img;

        void main()
        {
        }
    """, shader_stages)

    gen('declaration-format-qualifier-missing', """\
        ${header(status)}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "Uniforms not qualified with "writeonly" must have a format
         *  layout qualifier."
         */
        ${qualifier} uniform image2D img;

        void main()
        {
        }
    """, product(shader_stages, [
        {'name': 'writeonly',
         'qualifier': 'writeonly',
         'status': 'pass'},
        {'name': 'readonly',
         'qualifier': 'readonly',
         'status': 'fail'},
        {'name': 'readwrite',
         'qualifier': '',
         'status': 'fail'}
    ]))

    gen('declaration-memory-qualifier-sampler', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "Only variables declared as image types [...] can be qualified
         *  with a memory qualifier. "
         */
        ${qualifier} uniform sampler2D s;

        void main()
        {
        }
    """, product(shader_stages, [
        {'name': 'readonly', 'qualifier': 'readonly'},
        {'name': 'writeonly', 'qualifier': 'writeonly'},
        {'name': 'coherent', 'qualifier': 'coherent'},
        {'name': 'volatile', 'qualifier': 'volatile'},
        {'name': 'restrict', 'qualifier': 'restrict'}
    ]))

    #
    # Test expressions involving images.
    #
    gen('expression-allowed', """\
        ${header('pass')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "Except for array indexing, structure field selection, and
         *  parentheses, images are not allowed to be operands in
         *  expressions."
         */
        layout(rgba32f) uniform ${image_type} imgs[2];
        uniform vec4 y;

        out vec4 color;

        void main()
        {
                color = y + imageLoad((imgs[1]) ${image_addr()});
        }
    """, product(image_types[:1], shader_stages))

    gen('expression', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "Except for array indexing, structure field selection, and
         *  parentheses, images are not allowed to be operands in
         *  expressions."
         */
        layout(rgba32f) uniform image2D img;

        void main()
        {
                ${expression};
        }
    """, product(shader_stages, [
        {'name': 'arithmetic-1', 'expression': '-img'},
        {'name': 'arithmetic-2', 'expression': 'img + img'},
        {'name': 'arithmetic-3', 'expression': 'img - img'},
        {'name': 'arithmetic-4', 'expression': 'img * img'},
        {'name': 'arithmetic-5', 'expression': 'img / img'},
        {'name': 'arithmetic-6', 'expression': '++img'},
        {'name': 'arithmetic-7', 'expression': '--img'},
        {'name': 'arithmetic-8', 'expression': 'img++'},
        {'name': 'arithmetic-9', 'expression': 'img--'},
        {'name': 'assignment-1', 'expression': 'img ^= img'},
        {'name': 'assignment-2', 'expression': 'img |= img'},
        {'name': 'assignment-3', 'expression': 'img = img'},
        {'name': 'assignment-4', 'expression': 'img += img'},
        {'name': 'assignment-5', 'expression': 'img -= img'},
        {'name': 'assignment-6', 'expression': 'img *= img'},
        {'name': 'assignment-7', 'expression': 'img /= img'},
        {'name': 'assignment-8', 'expression': 'img %= img'},
        {'name': 'assignment-9', 'expression': 'img <<= img'},
        {'name': 'assignment-10', 'expression': 'img >>= img'},
        {'name': 'assignment-11', 'expression': 'img &= img'},
        {'name': 'binary-1', 'expression': '~img'},
        {'name': 'binary-2', 'expression': 'img << img'},
        {'name': 'binary-3', 'expression': 'img >> img'},
        {'name': 'binary-4', 'expression': 'img & img'},
        {'name': 'binary-5', 'expression': 'img | img'},
        {'name': 'binary-6', 'expression': 'img ^ img'},
        {'name': 'conversion-1', 'expression': 'float(img)'},
        {'name': 'conversion-2', 'expression': 'uint(img)'},
        {'name': 'conversion-3', 'expression': 'bool(img)'},
        {'name': 'conversion-4', 'expression': 'image1D(img)'},
        {'name': 'field-selection', 'expression': 'img.x'},
        {'name': 'function-call', 'expression': 'img()'},
        {'name': 'logical-1', 'expression': '!img'},
        {'name': 'logical-2', 'expression': 'img && img'},
        {'name': 'logical-3', 'expression': 'img || img'},
        {'name': 'logical-4', 'expression': 'img ^^ img'},
        {'name': 'relational-1', 'expression': 'img == img'},
        {'name': 'relational-2', 'expression': 'img != img'},
        {'name': 'relational-3', 'expression': 'img < img'},
        {'name': 'relational-4', 'expression': 'img > img'},
        {'name': 'relational-5', 'expression': 'img <= img'},
        {'name': 'relational-6', 'expression': 'img >= img'},
        {'name': 'selection', 'expression': 'true ? img : img'},
        {'name': 'subscript', 'expression': 'img[0]'}
    ]))

    #
    # Test passing of image variables in function calls.
    #
    gen('call-argument-qualifiers', """\
        ${header(status)}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "The values of image variables qualified with 'coherent',
         *  'volatile', 'restrict', 'readonly', or 'writeonly' may not be
         *  passed to functions whose formal parameters lack such
         *  qualifiers. [...] It is legal to have additional qualifiers on a
         *  formal parameter, but not to have fewer."
         */
        layout(rgba32f) ${actual_qualifier} uniform image2D x;

        void f(${formal_qualifier} image2D y)
        {
        }

        void main()
        {
            f(x);
        }
    """, product(shader_stages, [
        {'name': 'allowed-volatile',
         'actual_qualifier': 'volatile',
         'formal_qualifier': 'volatile coherent',
         'status': 'pass'},
        {'name': 'allowed-coherent',
         'actual_qualifier': 'coherent',
         'formal_qualifier': 'volatile coherent',
         'status': 'pass'},
        {'name': 'allowed-restrict',
         'actual_qualifier': 'restrict',
         'formal_qualifier': 'volatile restrict',
         'status': 'pass'},
        {'name': 'allowed-readonly',
         'actual_qualifier': 'readonly',
         'formal_qualifier': 'restrict readonly',
         'status': 'pass'},
        {'name': 'allowed-writeonly',
         'actual_qualifier': 'writeonly',
         'formal_qualifier': 'volatile writeonly',
         'status': 'pass'},
        {'name': 'disallowed-volatile',
         'actual_qualifier': 'volatile',
         'formal_qualifier': '',
         'status': 'fail'},
        {'name': 'disallowed-coherent',
         'actual_qualifier': 'coherent',
         'formal_qualifier': 'restrict',
         'status': 'fail'},
        {'name': 'disallowed-restrict',
         'actual_qualifier': 'restrict',
         'formal_qualifier': '',
         'status': 'fail'},
        {'name': 'disallowed-readonly',
         'actual_qualifier': 'readonly',
         'formal_qualifier': 'restrict writeonly',
         'status': 'fail'},
        {'name': 'disallowed-writeonly',
         'actual_qualifier': 'writeonly',
         'formal_qualifier': 'volatile readonly',
         'status': 'fail'}
    ]))

    gen('call-argument-type', """\
        ${header('pass' if image_type == 'image2D' else 'fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "As function parameters, images may only be passed to [arguments]
         *  of matching type."
         */
        layout(rgba32f) uniform image2D x;

        void f(${image_type} y)
        {
        }

        void main()
        {
            f(x);
        }
    """, product(image_types, shader_stages))

    #
    # Test the language built-in constants and functions.
    #
    gen('builtin-constants', """\
        ${header('pass')}
        /*
         * Check that the builtin constants defined by the extension
         * are present.
         */
        out ivec4 color;

        void main()
        {
            color.x = gl_MaxImageUnits +
                    gl_MaxCombinedImageUnitsAndFragmentOutputs +
                    gl_MaxImageSamples +
                    gl_MaxVertexImageUniforms +
                    gl_MaxGeometryImageUniforms +
                    gl_MaxFragmentImageUniforms +
                    gl_MaxCombinedImageUniforms;
        }
    """, shader_stages)

    gen('builtin-image-argument-mismatch', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "Atomic memory operations are supported on only [...]: an image
         *  variable with signed integer components (iimage*) format qualifier
         *  of 'r32i', or an image variable with unsigned integer components
         *  (uimage*) and format qualifier of 'r32ui'."
         *
         * Call an atomic built-in with a floating point image data type.
         */
        layout(r32f) uniform ${image_type} img;

        void main()
        {
            ${image_builtin}(img ${image_addr()} ${image_args('float')});
        }
    """, product(image_atomic_builtins, image_types[:1], shader_stages))

    gen('builtin-data-argument-mismatch', """\
        ${header('fail')}
        /*
         * Call a signed integer atomic built-in with a mismatching
         * argument data type.
         */
        layout(r32i) uniform i${image_type} img;

        void main()
        {
            ${image_builtin}(img ${image_addr()} ${image_args('uint')});
        }
    """, product(image_store_builtin + image_atomic_builtins,
                 image_types[:1], shader_stages))

    gen('builtin-address-argument-mismatch', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "The 'IMAGE_INFO' placeholder is replaced by one of the following
         *  parameter lists:
         *       gimage1D image, int coord
         *       gimage2D image, ivec2 coord
         *       gimage3D image, ivec3 coord
         *       gimage2DRect image, ivec2 coord
         *       gimageCube image, ivec3 coord
         *       gimageBuffer image, int coord
         *       gimage1DArray image, ivec2 coord
         *       gimage2DArray image, ivec3 coord
         *       gimageCubeArray image, ivec3 coord
         *       gimage2DMS image, ivec2 coord, int sample
         *       gimage2DMSArray image, ivec3 coord, int sample"
         *
         * Pass an argument as address coordinate that doesn't match the
         * dimensionality of the specified image.
         */
        layout(r32i) uniform i${image_type} img;

        void main()
        {
            ${image_builtin}(img ${image_addr(fail = True)} ${image_args('int')});
        }
    """, product(image_load_builtin + image_store_builtin + image_atomic_builtins,
                 image_types, shader_stages))

    gen('builtin-qualifier-mismatch-readonly', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "It is an error to pass an image variable qualified with 'readonly'
         *  to imageStore() or other built-in functions that modify image
         *  memory."
         *
         * Call a built-in function on a readonly qualified image.
         */
        layout(r32i) readonly uniform i${image_type} img;

        void main()
        {
            ${image_builtin}(img ${image_addr()} ${image_args('int')});
        }
    """, product(image_store_builtin + image_atomic_builtins,
                 image_types[:1], shader_stages))

    gen('builtin-qualifier-mismatch-writeonly', """\
        ${header('fail')}
        /*
         * From the ARB_shader_image_load_store spec:
         *
         * "It is an error to pass an image variable qualified with 'writeonly'
         *  to imageLoad() or other built-in functions that read image
         *  memory."
         *
         * Call a built-in function on a writeonly qualified image.
         */
        layout(r32i) writeonly uniform i${image_type} img;

        void main()
        {
            ${image_builtin}(img ${image_addr()} ${image_args('int')});
        }
    """, product(image_load_builtin + image_atomic_builtins,
                 image_types[:1], shader_stages))

    gen('builtin-memory-barrier', """\
        ${header('pass')}
        void main()
        {
            memoryBarrier();
        }
    """, shader_stages)


if __name__ == '__main__':
    main()



# coding=utf-8
# Copyright 2014-2016 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Piglit integrations for Crucible Test Suite.

upstream: https://gitlab.freedesktop.org/mesa/crucible/

"""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)
import os
import six
import subprocess
import tempfile

try:
    from lxml import etree
except ImportError:
    import xml.etree.cElementTree as etree

from framework import grouptools, backends, exceptions
from framework import status
from framework.core import PIGLIT_CONFIG
from framework.profile import TestProfile, Test

__all__ = ['profile']

crucible_bin = os.environ.get('PIGLIT_CRUCIBLE_BIN', None)
if crucible_bin is None:
    crucible_bin = PIGLIT_CONFIG.safe_get('crucible', 'bin')

if crucible_bin is None:
    raise exceptions.PiglitFatalError(
        'Cannot get "PIGLIT_CRUCIBLE_BIN" or conf value "crucible:bin"')


class CrucibleTest(Test):
    """Test representation for Crucible"""
    def __init__(self, case_name):
        self.__out_xml = tempfile.NamedTemporaryFile(delete=True).name
        command = [crucible_bin, 'run',
                   '--junit-xml={}'.format(self.__out_xml), case_name]
        self._case = case_name
        super(CrucibleTest, self).__init__(command)

    def interpret_result(self):
        try:
            test = backends.junit.REGISTRY.load(self.__out_xml, 'none')
            result = test.get_result(next(six.iterkeys(test.tests)))
            self.result.result = result.name
            super(CrucibleTest, self).interpret_result()
        except etree.ParseError:
            # This error is what cElementTree will generate, and is the parent
            # of what lxml will generate.
            self.result.result = status.CRASH
        finally:
            os.remove(self.__out_xml)

def gen_caselist_txt(bin_):
    with open('crucible.txt', 'w') as d:
        subprocess.check_call(
            [bin_, 'ls-tests'],
            stdout=d, stderr=d)
    assert os.path.exists('crucible.txt')
    return 'crucible.txt'

def _populate_profile():
    profile = TestProfile()
    case_file = gen_caselist_txt(crucible_bin)
    with open(case_file, 'r') as caselist_file:
        for i, line in enumerate(caselist_file):
            case = line.rstrip()
            piglit_name = case.replace('.', grouptools.SEPARATOR)
            profile.test_list[piglit_name] = CrucibleTest(case)
    os.remove('crucible.txt')
    return profile

profile = _populate_profile()



# encoding=utf-8
# Copyright © 2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Generate ARBvp tests.

Based on a bash generator that performs the same function written by Ian
Romanick.

"""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)
import os

from templates import template_dir
from modules import utils

TEMPLATES = template_dir(os.path.basename(os.path.splitext(__file__)[0]))


def main():
    """Main function. Generates tests."""
    dirname = os.path.join('asmparsertest', 'shaders', 'ARBvp1.0')
    utils.safe_makedirs(dirname)

    targets_1 = ["1D", "2D", "3D", "CUBE", "RECT", "SHADOW1D", "SHADOW2D"]

    # The ordering in this loop is wierd, it's an artifact of the growth of the
    # original script and is required to ensure that the names of the tests
    # don't change
    for inst in ["TEX", "TXB", "TXD", "TXF", "TXL", "TXP", "TXQ"]:
        i = 1
        template = TEMPLATES.get_template('arbvp.mako')
        for target in targets_1:
            fname = os.path.join(dirname,
                                 "{}-{:0>2d}.txt".format(inst.lower(), i))
            with open(fname, 'w') as f:
                f.write(template.render_unicode(target=target, inst=inst))
            print(fname)
            i += 1

        template = TEMPLATES.get_template('nvvp3.mako')
        for target in targets_1:
            fname = os.path.join(dirname,
                                 "{}-{:0>2d}.txt".format(inst.lower(), i))
            with open(fname, 'w') as f:
                f.write(template.render_unicode(target=target, inst=inst))
            print(fname)
            i += 1

        template = TEMPLATES.get_template('nvvp3_2.mako')
        for target in ["CUBE", "RECT"]:
            fname = os.path.join(dirname,
                                 "{}-{:0>2d}.txt".format(inst.lower(), i))
            with open(fname, 'w') as f:
                f.write(template.render_unicode(target=target, inst=inst))
            print(fname)
            i += 1

        template = TEMPLATES.get_template('nvvp3.mako')
        fname = os.path.join(dirname, "{}-{:0>2d}.txt".format(inst.lower(), i))
        with open(fname, 'w') as f:
            f.write(template.render_unicode(target="SHADOWRECT", inst=inst))
        print(fname)
        i += 1

        template = TEMPLATES.get_template('nvvp3_2.mako')
        for target in ["SHADOW1D", "SHADOW2D", "SHADOWRECT"]:
            fname = os.path.join(dirname,
                                 "{}-{:0>2d}.txt".format(inst.lower(), i))
            with open(fname, 'w') as f:
                f.write(template.render_unicode(target=target, inst=inst))
            print(fname)
            i += 1


if __name__ == '__main__':
    main()



# coding=utf-8
# Copyright (c) 2015-2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Generate tests for variable index writes.

This creates a TestParams object for each invocation (using a factory to reduce
the number of duplicate objects), and then passes that into a mako template.
The template then formats that information into a shader_test for either the
fragment shader stage or the vertex shader stage.

"""

from __future__ import (
    print_function, absolute_import, division, unicode_literals
)
import copy
import itertools
import os

from six.moves import range  # pylint: disable=redefined-builtin

from modules import utils, glsl
from templates import template_dir

_TEMPLATES = template_dir(os.path.basename(os.path.splitext(__file__)[0]))
_VS_TEMPLATE = _TEMPLATES.get_template('vs.shader_test.mako')
_FS_TEMPLATE = _TEMPLATES.get_template('fs.shader_test.mako')
_DIRNAME = os.path.join('spec', 'glsl-{}', 'execution', 'variable-indexing')


class TestParams(object):
    """Object representing all of the parameters of a single test instance.

    Provides all of the values using lazy properties, which tie in with the
    ParamsFactory to store all of the values, speeding up run times.

    """
    def __init__(self, mode, array_dim, matrix_dim, index_value, col,
                 value_type, glsl_version):
        # pylint: disable=too-many-arguments
        assert array_dim in [0, 3]
        assert matrix_dim in [2, 3, 4]

        self.mode = mode
        self.array_dim = array_dim
        self.matrix_dim = matrix_dim
        self.index_value = index_value
        self.col = '[{}]'.format(col)
        self.value_type = value_type
        self.version = glsl.Version(glsl_version)

    @utils.lazy_property
    def varying_comps(self):
        if self.array_dim != 0:
            return self.matrix_dim**2 * self.array_dim
        else:
            return self.matrix_dim**2

    @utils.lazy_property
    def base_type(self):
        if int(self.version) >= 120:
            return 'mat{0}x{0}'.format(self.matrix_dim)
        else:
            return 'mat{}'.format(self.matrix_dim)

    @utils.lazy_property
    def type(self):
        if self.array_dim != 0 and int(self.version) >= 120:
            return '{}[{}]'.format(self.base_type, self.array_dim)
        else:
            return self.base_type

    @utils.lazy_property
    def dim(self):
        if self.array_dim != 0 and int(self.version) < 120:
            return '[{}]'.format(self.array_dim)
        else:
            return ''

    @utils.lazy_property
    def row(self):
        if self.value_type == 'float':
            return '[row]'
        else:
            return ''

    @utils.lazy_property
    def idx(self):
        if self.array_dim != 0:
            return '[{}]'.format(self.index_value)
        else:
            return ''

    @utils.lazy_property
    def test_vec(self):
        if self.matrix_dim == 2:
            return ["0.803161418975390", "0.852987140792140"]
        elif self.matrix_dim == 3:
            return ["0.681652305322399", "0.210426138878113",
                    "0.185916924650237"]
        elif self.matrix_dim == 4:
            return ["0.0394868046587045", "0.8922408276905568",
                    "0.3337495624366961", "0.8732295730825839"]

    @utils.lazy_property
    def test_exp(self):
        if self.matrix_dim == 2:
            return ["0.708718134966688", "1.452243795483797"]
        elif self.matrix_dim == 3:
            return ["0.610649606928364", "0.711906885823636",
                    "0.312244778977868"]
        elif self.matrix_dim == 4:
            return ["1.03935908892461", "1.18846180713529", "1.10078681232072",
                    "1.72434439561820"]

    @utils.lazy_property
    def test_mat(self):
        if self.matrix_dim == 2:
            return [["0.241498998195656", "0.861223395812970"],
                    ["0.603473877011433", "0.891622340451180"]]
        elif self.matrix_dim == 3:
            return [
                ["0.493944462129466", "0.722190133917966", "0.239853948232558"],
                ["0.550143078409278", "0.591962645398579", "0.467616286531193"],
                ["0.850846377186973", "0.511303112962423", "0.270815003356504"]
            ]
        elif self.matrix_dim == 4:
            return [["0.922040144261674", "0.158053783109488",
                     "0.357016429866574", "0.836368810383957"],
                    ["0.560251913703792", "0.171634921595771",
                     "0.602494709909111", "0.693273570571311"],
                    ["0.350720358904176", "0.912192627475775",
                     "0.688544081259531", "0.913891056231967"],
                    ["0.442058176039301", "0.829835836794679",
                     "0.365674411003021", "0.879197364462782"]]

    @utils.lazy_property
    def test_sizes(self):
        if self.array_dim == 0:
            return [1]
        elif self.index_value == 'index':
            return list(range(1, self.array_dim + 1))
        else:
            return [2]

    @utils.lazy_property
    def test_columns(self):
        if self.col == '[col]':
            return list(range(1, self.matrix_dim + 1))
        return [2]

    @utils.lazy_property
    def test_rows(self):
        if self.value_type == 'float':
            return list(range(1, self.matrix_dim + 1))
        return [1]

    @utils.lazy_property
    def test_type(self):
        # shader_runner always uses matDxD format
        return 'mat{0}x{0}'.format(self.matrix_dim)

    def test_matrix(self, column, row):
        """Generate the matrix used in a test section.

        This will take the matrix used by the test, and replace specific values
        with sentinal values, and return the matrix as a string.

        """
        bad = ['666.0', '777.0', '888.0', '999.0']
        mat = copy.deepcopy(self.test_mat)

        if self.value_type == 'float':
            mat[column][row] = bad[0]
        else:
            mat[column] = bad[0:self.matrix_dim]

        ret = ''
        for c in mat:
            ret += ' '.join(c)
            ret += ' '
        return ret

    @utils.lazy_property
    def formated_version(self):
        # Note: GLSLVersion.float() does division by 100
        return '{:.2f}'.format(float(self.version))


class ParamsFactory(object):  # pylint: disable=too-few-public-methods
    """A factory class that provides TestParam objects.

    This cuts the number of new objects created by roughly 3/5.

    """
    def __init__(self):
        self.__stored = {}

    def get(self, *args):
        args = tuple(args)
        try:
            return self.__stored[args]
        except KeyError:
            params = TestParams(*args)
            self.__stored[args] = params
            return params


def make_vs(name, params):
    """Create a vertex shader test."""
    dirname = _DIRNAME.format(params.formated_version)
    utils.safe_makedirs(dirname)
    with open(os.path.join(dirname, name), 'w') as f:
        f.write(_VS_TEMPLATE.render_unicode(params=params))
    print(name)


def make_fs(name, params):
    """Create a fragment shader test."""
    dirname = _DIRNAME.format(params.formated_version)
    utils.safe_makedirs(dirname)
    with open(os.path.join(dirname, name), 'w') as f:
        f.write(_FS_TEMPLATE.render_unicode(params=params))
    print(name)


def main():
    """The main function."""
    # Various choices that can be put together to produce a single test.
    iter_ = itertools.product(
        ['110', '120'],       # GLSL versions
        [0, 3],               # Array dimensions
        [2, 3, 4],            # Matrix dimensions
        ['varying', 'temp'],  # modes
        ['col', 1],           # columns
        ['fs', 'vs'],         # shader stages
    )

    factory = ParamsFactory()

    # This can be filled in to produce the file name for the test.
    # Note that idx, col, row, and arr will need to have a '-' added to the end
    # of the value if it is not empty
    name = '{stage}-{mode}-{arr}mat{matrix_dim}-{idx}{col}{row}wr.shader_test'

    for v, a, d, m, c, s in iter_:
        for t in ['float', 'vec{}'.format(d)]:
            if s == 'vs':
                func = make_vs
            elif s == 'fs':
                if m == 'varying':
                    # Fragment shaders cannot write varyings
                    continue
                func = make_fs

            if a != 0:
                arr = 'array-'

                func(
                    name.format(stage=s,
                                mode=m,
                                matrix_dim=d,
                                arr=arr,
                                idx='',
                                col='col-' if c == 'col' else '',
                                row='row-' if t == 'float' else ''),
                    factory.get(m, a, d, 1, c, t, v))
            else:
                arr = ''

            func(
                name.format(stage=s,
                            mode=m,
                            matrix_dim=d,
                            arr=arr,
                            idx='index-' if a != 0 else '',
                            col='col-' if c == 'col' else '',
                            row='row-' if t == 'float' else ''),
                factory.get(m, a, d, 'index', c, t, v))


if __name__ == '__main__':
    main()



# coding=utf-8
# Copyright (c) 2015-2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Generate tests for running the generators.

This needs to be compatible with both python2 and python3.

Tests that take more than ~10 seconds should be marked as "slow" and tests that
take more than ~30 seconds should be marked as "very_slow".
"""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)
import importlib
try:
    import mock
except ImportError:
    from unittest import mock

import pytest


@pytest.mark.parametrize('name', [
    'gen_builtin_packing_tests',
    'gen_builtin_uniform_tests_fp64',
    'gen_cl_common_builtins',
    'gen_cl_int_builtins',
    'gen_cl_math_builtins',
    'gen_cl_relational_builtins',
    'gen_cl_shuffle2_builtins',
    'gen_cl_shuffle_builtins',
    'gen_cl_store_tests',
    'gen_cl_vload_tests',
    'gen_cl_vstore_tests',
    'gen_const_builtin_equal_tests',
    'gen_constant_array_size_tests_fp64',
    'gen_conversion',
    'gen_extensions_defined',
    'gen_flat_interpolation_qualifier',
    'gen_inout_fp64',
    'gen_interpolation_tests',
    'gen_non-lvalue_tests',
    'gen_outerproduct_invalid_params',
    'gen_outerproduct_tests',
    'gen_shader_bit_encoding_tests',
    'gen_shader_framebuffer_fetch_tests',
    'gen_shader_image_load_store_tests',
    'gen_shader_image_nv_image_formats_tests',
    'gen_shader_intel_conservative_rasterization',
    'gen_shader_precision_tests',
    'gen_tcs_input_tests',
    'gen_tes_input_tests',
    'gen_texture_lod_tests',
    'gen_texture_query_lod_tests',
    'gen_uniform_initializer_tests',
    'gen_variable_index_read_tests',
    'gen_variable_index_write_tests',
    'gen_vp_tex',
    'interpolation-qualifier-built-in-variable',
    pytest.mark.slow('gen_builtin_uniform_tests'),
    pytest.mark.slow('gen_constant_array_size_tests'),
    pytest.mark.very_slow('gen_vs_in_fp64'),
])
def test_generators(name, tmpdir):
    """Teat each generator."""
    mod = importlib.import_module(name)
    tmpdir.chdir()

    # Some tests do checks for sys.argv, so mock that out. Also mock out
    # print since we don't want a giant list of tests printed on an error.
    with mock.patch('sys.argv', [name]), \
            mock.patch.object(mod, 'print', mock.Mock(), create=True):
        mod.main()



# coding=utf-8
from __future__ import print_function, division, absolute_import
import os

import six

__all__ = ['gen', 'DATA_SIZES', 'MAX_VALUES', 'MAX', 'MIN', 'BMIN', 'BMAX',
           'SMIN', 'SMAX', 'UMIN', 'UMAX', 'TYPE', 'T', 'U', 'B']

_NUMERIC_TYPES = tuple(list(six.integer_types) + [float])


DATA_SIZES = {
    'char': 8,
    'uchar': 8,
    'short': 16,
    'ushort': 16,
    'int': 32,
    'uint': 32,
    'long': 64,
    'ulong': 64
}

# By default, just test what is part of the CL1.1 spec, leave vec3 for later
# VEC_WIDTHS = [2, 3, 4, 8, 16]
VEC_WIDTHS = [2, 4, 8, 16]

# ALL_WIDTHS = [1, 2, 3, 4, 8, 16]
ALL_WIDTHS = [1, 2, 4, 8, 16]

MIN_VALUES = {
    'char': -128,
    'uchar': 0,
    'short': -32768,
    'ushort': 0,
    'int': -2147483648,
    'uint': 0,
    'long': -9223372036854775808,
    'ulong': 0
}

MAX_VALUES = {
    'char': 127,
    'uchar': 255,
    'short': 32767,
    'ushort': 65535,
    'int': 2147483647,
    'uint': 4294967295,
    'long': 9223372036854775807,
    'ulong': 18446744073709551615
}

# Define placeholders to reduce magic number usage
MAX = 'MAX_VAL'
MIN = 'MIN_VAL'
BMIN = 'min_for_larger_type'
BMAX = 'max_for_larger_type'
SMIN = 'signed_min_for_type'
SMAX = 'signed_max_for_type'
UMIN = 'unsigned_min_for_type'
UMAX = 'unsigned_max_for_type'
TYPE = 'TYPE'
SIZE = 'SIZE'
TRUE = 'true_value_for_type' #1 for scalar, -1 for vector
NEGNAN = 'Negative NAN as a string, because float("-nan") just produces nan'

# Identity type list
T = {
    'char': 'char',
    'uchar': 'uchar',
    'short': 'short',
    'ushort': 'ushort',
    'int': 'int',
    'uint': 'uint',
    'long': 'long',
    'ulong': 'ulong'
}
# Signed type for each type
SIGNED = {
    'char': 'char',
    'uchar': 'char',
    'short': 'short',
    'ushort': 'short',
    'int': 'int',
    'uint': 'int',
    'long': 'long',
    'ulong': 'long'
}
# Unsigned type for each source type
U = {
    'char': 'uchar',
    'uchar': 'uchar',
    'short': 'ushort',
    'ushort': 'ushort',
    'int': 'uint',
    'uint': 'uint',
    'long': 'ulong',
    'ulong': 'ulong'
}
# Next larger type with same signedness
B = {
    'char': 'short',
    'uchar': 'ushort',
    'short': 'int',
    'ushort': 'uint',
    'int': 'long',
    'uint': 'ulong',
}


# vecSizes has the layout [in0width, ..., inNwidth] where outType width is
# assumed to match the width of the first input
def gen_kernel(f, fnName, inTypes, outTypes, vecSizes, typePrefix, outLoc='private'):
    f.write('kernel void test_' + typePrefix + str(vecSizes[0]) + '_' + fnName
            + (('_' + outLoc) if len(outTypes) > 1 else '')
            + '_' + inTypes[0] + '(global ' + outTypes[0] + '* out')
    for arg in range(1, len(outTypes)):
        f.write(', global '+outTypes[arg]+'* out'+str(arg))
    for arg in range(0, len(inTypes)):
        f.write(', global '+inTypes[arg]+'* in'+str(arg))
    f.write('){\n')

    for arg in range(1, len(outTypes)):
        f.write('  ' + outLoc + ' ' + outTypes[arg] + ('' if vecSizes[0] == 1 else str(vecSizes[0])));
        if (outLoc == 'global'):
            f.write(' *tmp' + str(arg) + ' = out' + str(arg) + ';\n');
        else:
            #FIXME: This assumes WG size of 1
            f.write(' tmp' + str(arg) + ';\n');

    suffix = ';'
    if (vecSizes[0] == 1):
        f.write('  out[get_global_id(0)] = ')
    else:
        f.write('  vstore'+str(vecSizes[0])+'(')
        suffix = ', get_global_id(0), out)' + suffix

    f.write(fnName+'(')
    suffix = ')' + suffix

    for arg in range(0, len(inTypes)):
        if (arg > 0):
            f.write(', ')
        # if scalar, don't print vload/vstore
        if (vecSizes[arg] == 1):
            f.write('in'+str(arg)+'[get_global_id(0)]')
        else:
            f.write('vload'+str(vecSizes[arg])+'(get_global_id(0), in'+str(arg)+')')
    for arg in range(1, len(outTypes)):
        if (outLoc == 'global'):
            f.write(', &tmp' + str(arg) + '[get_global_id(0)]')
        else:
            f.write(', &tmp' + str(arg))

    f.write(suffix + '\n')

    if (outLoc != 'global'):
        for arg in range(1, len(outTypes)):
            if (vecSizes[0] == 1):
                f.write('  out'+ str(arg) +'[get_global_id(0)] = tmp' + str(arg))
            else:
                f.write('  vstore' + str(vecSizes[0]) + '(tmp' + str(arg) + ', get_global_id(0), out' + str(arg) + ')')

            f.write(';\n')

    f.write('}\n\n')



def gen_kernel_1_arg(f, fnName, inType, outTypes, loc = 'private'):
    for vecSize in ALL_WIDTHS:
        gen_kernel(f, fnName, [inType], outTypes, [vecSize], '', loc)


#  2 argument kernel with input types that match their vector size
def gen_kernel_2_arg_same_size(f, fnName, inTypes, outTypes, loc = 'private'):
    for vecSize in ALL_WIDTHS:
        gen_kernel(f, fnName, inTypes, outTypes, [vecSize, vecSize],
                   '', loc)


#  2 argument kernel with 1 vector and one scalar input argument
def gen_kernel_2_arg_mixed_size(f, fnName, inTypes, outTypes):
    for vecSize in VEC_WIDTHS:
        gen_kernel(f, fnName, inTypes, outTypes, [vecSize, 1], 'tss_')


#  2 argument kernel with 1 vector and one scalar input argument with multiple
#    input data types
def gen_kernel_2_arg_mixed_sign(f, fnName, inTypes, outTypes):
    for vecSize in ALL_WIDTHS:
        gen_kernel(f, fnName, inTypes, outTypes, [vecSize, vecSize],
                   '')


#  3-argument built-in functions


def gen_kernel_3_arg_same_type(f, fnName, inTypes, outTypes):
    for vecSize in ALL_WIDTHS:
        gen_kernel(f, fnName, inTypes, outTypes,
                   [vecSize, vecSize, vecSize], ''
        )

def gen_kernel_3_arg_mixed_size_tss(f, fnName, inTypes, outTypes):
    for vecSize in VEC_WIDTHS:
        gen_kernel(f, fnName, inTypes, outTypes,
                   [vecSize, 1, 1], 'tss_')

def gen_kernel_3_arg_mixed_size_tts(f, fnName, inTypes, outTypes):
    for vecSize in VEC_WIDTHS:
        gen_kernel(f, fnName, inTypes, outTypes,
                   [vecSize, vecSize, 1], 'tts_')


def generate_kernels(f, dataType, fnName, fnDef):

    argTypes = getArgTypes(dataType, fnDef['arg_types'])

    # For len(argTypes), remember that this includes the output arg
    if (len(argTypes) == 2):
        gen_kernel_1_arg(f, fnName, argTypes[1], [argTypes[0]])
        return

    if (len(argTypes) == 3 and fnName != 'upsample'):
        if (getNumOutArgs(fnDef) == 2):
            gen_kernel_1_arg(f, fnName,
                             argTypes[2], [argTypes[0], argTypes[1]], 'private')
            gen_kernel_1_arg(f, fnName,
                             argTypes[2], [argTypes[0], argTypes[1]], 'local')
            gen_kernel_1_arg(f, fnName,
                             argTypes[2], [argTypes[0], argTypes[1]], 'global')
        else:
            gen_kernel_2_arg_same_size(f, fnName,
                                    [argTypes[1], argTypes[2]], [argTypes[0]])
        if (fnDef['function_type'] == 'tss'):
            gen_kernel_2_arg_mixed_size(f, fnName,
                                [argTypes[1], argTypes[2]], [argTypes[0]])
        return

    if (len(argTypes) == 4):
        if (getNumOutArgs(fnDef) == 2):
            gen_kernel_2_arg_same_size(f, fnName, [argTypes[2], argTypes[3]],
                                       [argTypes[0], argTypes[1]], 'private')
            gen_kernel_2_arg_same_size(f, fnName, [argTypes[2], argTypes[3]],
                                       [argTypes[0], argTypes[1]], 'local')
            gen_kernel_2_arg_same_size(f, fnName, [argTypes[2], argTypes[3]],
                                       [argTypes[0], argTypes[1]], 'global')
        else:
            gen_kernel_3_arg_same_type(f, fnName,
                       [argTypes[1], argTypes[2], argTypes[3]], [argTypes[0]])
        if (fnDef['function_type'] == 'tss'):
            gen_kernel_3_arg_mixed_size_tss(f, fnName,
                   [argTypes[1], argTypes[2], argTypes[3]], [argTypes[0]])
        if (fnDef['function_type'] == 'tts'):
            gen_kernel_3_arg_mixed_size_tts(f, fnName,
                   [argTypes[1], argTypes[2], argTypes[3]], [argTypes[0]])
        return

    if (fnName == 'upsample'):
        gen_kernel_2_arg_mixed_sign(f, fnName,
                                    [argTypes[1], argTypes[2]],
                                    [argTypes[0]])
        return

def getValue(type, val, isVector):
    # Check if val is a str, list, or value
    if (isinstance(val, str)):
        if (val == MIN):
            return MIN_VALUES[type]
        elif (val == MAX):
            return MAX_VALUES[type]
        elif (val == BMIN):
            return MIN_VALUES[B[type]]
        elif (val == BMAX):
            return MAX_VALUES[B[type]]
        elif (val == SMIN):
            return MIN_VALUES[SIGNED[type]]
        elif (val == SMAX):
            return MAX_VALUES[SIGNED[type]]
        elif (val == UMIN):
            return MIN_VALUES[U[type]]
        elif (val == UMAX):
            return MAX_VALUES[U[type]]
        elif (val == TYPE):
            return type
        elif (val == SIZE):
            return DATA_SIZES[type]
        elif (val == TRUE):
            if (isVector):
                return -1
            else:
                return 1
        elif (val == NEGNAN):
            return '-nan' #cl-program-tester translates this for us
        else:
            print('Unknown string value: ' + val + '\n')
    elif (isinstance(val, list)):
        # The list should be of the format: [op, arg1, ... argN] where op is a
        # Fn ref and arg[1-n] are either MIN/MAX or numbers (They could be
        # nested lists). The exception for arg1 is TYPE, which means to
        # substitute the data type

        # Evaluate the value of the requested function and arguments
        # TODO: Change to varargs calls after unshifting the first list element
        if (callable(val[0])):
            if (len(val) == 2):
                return (val[0])(getValue(type, val[1], isVector))
            elif (len(val) == 3):
                return (val[0])(getValue(type, val[1], isVector), getValue(type, val[2], isVector))
            elif (len(val) == 4):
                return (val[0])(getValue(type, val[1], isVector), getValue(type, val[2], isVector),
                            getValue(type, val[3], isVector))
            else:
                return (val[0])(getValue(type, val[1], isVector), getValue(type, val[2], isVector),
                            getValue(type, val[3], isVector), getValue(type, val[4], isVector))
        else:
             return map(lambda x: getValue(type, x, isVector), val);

    # At this point, we should have been passed a number
    if (isinstance(val, _NUMERIC_TYPES)):
        return val

    print('Invalid value '+repr(val)+' encountered in getValue\n')



def getStrVal(type, val, isVector):
    return " ".join(map(str, getValue(type, val, isVector)))


def getArgType(baseType, argType):
    # If the argType is a string, it's a literal data type... return it
    if (isinstance(argType, str)):
        return argType
    # otherwise it's a list to pick from
    return argType[baseType]


def getArgTypes(baseType, argTypes):
    ret = []
    for argType in argTypes:
        ret.append(getArgType(baseType, argType))
    return ret

def isFloatType(t):
    return t not in U

def getNumOutArgs(functionDef):
    if 'num_out_args' in functionDef:
        return functionDef['num_out_args']
    else:
        return 1

def isOutArg(functionDef, argIdx):
    return argIdx < getNumOutArgs(functionDef)

# Print a test with all-vector inputs/outputs and/or mixed vector/scalar args
def print_test(f, fnName, argType, functionDef, tests, numTests, vecSize, fntype):
    # If the test allows mixed vector/scalar arguments, handle the case with
    # only vector arguments through a recursive call.
    if (fntype == 'tss' or fntype == 'tts'):
        print_test(f, fnName, argType, functionDef, tests, numTests, vecSize,
                   'ttt')

    # The tss && vecSize==1 case is handled in the non-tss case.
    if ((fntype != 'ttt') and vecSize == 1):
        return

    # If we're handling mixed vector/scalar input widths, the kernels have
    # different names than when the vector widths match
    tssStr = fntype + '_' if (fntype != 'ttt') else ''

    argTypes = getArgTypes(argType, functionDef['arg_types'])
    argCount = len(argTypes)
    tolerance = functionDef['tolerance'] if 'tolerance' in functionDef else 0

    # Write the test header
    f.write('[test]\n' + 'name: ' + tssStr + fnName + ' ' + argType
            + str(vecSize) + '\n' + 'kernel_name: test_' + tssStr + str(vecSize)
            + '_' + fnName + '_' + argType + '\n' + 'global_size: '
            + str(numTests) + ' 0 0\n\n'
    )

    # For each argument, write a line containing its type, index, and values
    for arg in range(0, argCount):
        argInOut = ''
        argVal = getStrVal(argType, tests[arg], (vecSize > 1))
        if isOutArg(functionDef, arg):
            argInOut = 'arg_out: '
        else:
            argInOut = 'arg_in: '

        # The output argument and first tss argument are vectors, any that
        # follow are scalar. If !tss, then everything has a matching vector
        # width
        if (fntype == 'ttt' or (arg < 2 and fntype == 'tss') or (arg < 3 and fntype == 'tts')):
            f.write(argInOut + str(arg) + ' buffer ' + argTypes[arg] +
                    '[' + str(numTests * vecSize) + '] ' +
                    ''.join(map(lambda x: (x + ' ') * vecSize, argVal.split()))
            )
            if isOutArg(functionDef, arg) :
                f.write(' tolerance {0} '.format(tolerance))
                # Use ulp tolerance for float types
                if isFloatType(argTypes[arg]):
                    f.write('ulp')
            f.write('\n')
        else:
            argInOut = 'arg_in: '
            f.write(argInOut + str(arg) + ' buffer ' + argTypes[arg] + '[' +
                    str(numTests) + '] ' + argVal + '\n'
            )

    # Blank line between tests for formatting reasons
    f.write('\n')


def gen(types, minVersions, functions, testDefs, dirName):
    # Create the output directory if required
    if not os.path.exists(dirName):
        try:
            os.makedirs(dirName)
        except OSError as e:
            if e.errno == 17:  # file exists
                pass
            raise

    # Loop over all data types being tested. Create one output file per data
    # type
    for dataType in types:
        for fnName in functions:
            # Merge all of the generic/signed/unsigned/custom test definitions
            if (dataType, fnName) not in testDefs:
                continue
            functionDef = testDefs[(dataType, fnName)]

            # Check if the function actually exists for this data type
            if (not functionDef.keys()):
                continue

            clcVersionMin = minVersions[fnName]

            fileName = 'builtin-' + dataType + '-' + fnName + '-' + \
                str(float(clcVersionMin)/10)+'.generated.cl'

            fileName = os.path.join(dirName, fileName)

            with open(fileName, 'w') as f:
                print(fileName)
                # Write the file header
                f.write('/*!\n' +
                        '[config]\n' +
                        'name: Test '+dataType+' '+fnName+' built-in on CL 1.1\n' +
                        'clc_version_min: '+str(clcVersionMin)+'\n' +
                        'dimensions: 1\n'
                )
                if (dataType == 'double'):
                    f.write('require_device_extensions: cl_khr_fp64\n')

                # Blank line  to provide separation between config header and tests
                f.write('\n')

                # Write all tests for the built-in function
                tests = functionDef['values']
                argCount = len(functionDef['arg_types'])
                fnType = functionDef['function_type']

                outputValues = tests[0]
                numTests = len(outputValues)

                # Handle all available scalar/vector widths
                sizes = sorted(VEC_WIDTHS)
                sizes.insert(0, 1)  # Add 1-wide scalar to the vector widths
                for vecSize in sizes:
                    if (getNumOutArgs(functionDef) == 1):
                        print_test(f, fnName, dataType, functionDef, tests,
                                   numTests, vecSize, fnType)
                    else:
                        for loc in ['_private', '_local', '_global']:
                            print_test(f, fnName + loc, dataType, functionDef, tests,
                                       numTests, vecSize, fnType)

                # Terminate the header section
                f.write('!*/\n\n')

                if (dataType == 'double'):
                    f.write('#pragma OPENCL EXTENSION cl_khr_fp64 : enable\n\n')

                # Generate the actual kernels
                generate_kernels(f, dataType, fnName, functionDef)



# coding=utf-8
# Copyright (c) 2015-2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Tests for the dmesg module.

This module makes extensive use of mock to avoid actually calling into dmesg,
which allows us to test all classes on all platforms, including windows.

"""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)
import collections
import re
try:
    import mock
except ImportError:
    from unittest import mock

import pytest
import six

from framework import dmesg
from framework import status
from framework import results
from framework import exceptions

from . import skip

# pylint: disable=invalid-name,no-self-use


class _DmesgTester(dmesg.BaseDmesg):
    """Test Dmesg class. stubs update_dmesg and __init__"""

    def __init__(self):
        super(_DmesgTester, self).__init__()
        self._new_messages = ['some', 'new', 'messages']

    def update_dmesg(self, *args, **kwargs):
        pass


class TestBaseDmesg(object):
    """Tests for the BaseDmesg class."""

    result = None

    @classmethod
    def setup_class(cls):
        cls.dmesg = _DmesgTester()

    def setup(self):
        self.result = results.TestResult()
        self.result.dmesg = mock.sentinel.dmesg

    def test_update_result_dmesg(self):
        """dmesg.BaseDmesg.update_result: records new dmesg content in result"""
        self.dmesg.update_result(self.result)
        assert self.result.dmesg is not mock.sentinel.dmesg

    @pytest.mark.parametrize(
        "initial,expected",
        [
            (status.PASS, status.DMESG_WARN),
            (status.WARN, status.DMESG_FAIL),
            (status.FAIL, status.DMESG_FAIL),
            (status.CRASH, status.CRASH),
            (status.SKIP, status.SKIP),
            (status.NOTRUN, status.NOTRUN),
            (status.TIMEOUT, status.TIMEOUT),
        ],
        ids=six.text_type)
    def test_update_result_status(self, initial, expected):
        """Test that when update_result is called status change when they
        should, and don't when they shouldn't.
        """
        self.result.result = initial
        self.dmesg.update_result(self.result)
        assert self.result.result is expected

    @pytest.mark.parametrize(
        "initial,expected",
        [
            (status.PASS, status.DMESG_WARN),
            (status.WARN, status.DMESG_FAIL),
            (status.FAIL, status.DMESG_FAIL),
            (status.CRASH, status.CRASH),
            (status.SKIP, status.SKIP),
            (status.NOTRUN, status.NOTRUN),
            (status.TIMEOUT, status.TIMEOUT),
        ],
        ids=six.text_type)
    def test_update_result_subtests(self, initial, expected):
        """Test that when update_result is called subtest statuses change when
        they should, and don't when they shouldn't.
        """
        self.result.subtests['foo'] = initial
        self.result.subtests['bar'] = initial

        self.dmesg.update_result(self.result)

        assert self.result.subtests['foo'] is expected
        assert self.result.subtests['bar'] is expected

    def test_update_result_regex_no_match(self):
        """dmesg.BaseDmesg.update_result: if no regex matches don't change
        status.
        """
        self.dmesg.regex = re.compile(r'nomatchforthisreally')
        self.result.result = status.PASS

        self.dmesg.update_result(self.result)

        assert self.result.result is status.PASS

    def test_update_result_regex_match(self):
        """dmesg.BaseDmesg.update_result: if regex matches change status."""
        self.dmesg.regex = re.compile(r'.*')
        self.result.result = status.PASS

        self.dmesg.update_result(self.result)

        assert self.result.result is status.DMESG_WARN


class TestLinuxDmesgTimestamps(object):
    """Tests for the LinuxDmesg.__init__ timestampe detection.

    The linux path will try to open /proc/config.gz and look for
    CONFIG_PRINTK_TIME=y, there are a couple of things that can go wrong it
    should recover from, in both of the following cases it's expected to go
    on and fall back to less exact checking.

    The first is that there is no /proc/config.gz in that case we'll get a
    OSError in python2 or it's descendent FileNotFoundError in python 3.

    The second is that it could get an IOError in python 2.x or
    PermissionError in python 3.x, if the file cannot be read.

    The fallback path will try read dmesg and see if there's evidence of a
    timestamp, or warn that it can't find anything.

    """
    def test_warn(self, mocker):
        """Test that if /proc/config is not available, and there are no values
        to check in _last_message a RuntimeWarning should be issued.
        """
        # Mocking this will prevent LinuxDmesg._last_message from being
        # updated, which will force us down the warn path
        mocker.patch('framework.dmesg.LinuxDmesg.update_dmesg')

        # OSError was picked because it will work for both python2 and python3
        # (FileNotFoundError is a descendent of OSError)
        mocker.patch('framework.dmesg.gzip.open', side_effect=OSError)

        with pytest.warns(RuntimeWarning):
            dmesg.LinuxDmesg()

    # Implementation notes:
    #
    # It would seem like a parametrized test would be what we want here, since
    # these tests share so much state, but FileNotFoundError and
    # PermissionError don't exist in python 2.x, and the parametrizer will
    # cause an error. Using a class with a shared method is the next best
    # solution.
    @staticmethod
    def _do_test(error, mocker):
        # Mocking this will prevent LinuxDmesg._last_message from being
        # updated, which will force us down the warn path, which is convenient
        # for assertion puproses.
        mocker.patch('framework.dmesg.LinuxDmesg.update_dmesg')
        mocker.patch('framework.dmesg.gzip.open', side_effect=error)

        with pytest.warns(RuntimeWarning):
            dmesg.LinuxDmesg()

    @skip.PY3
    def test_config_oserror(self, mocker):
        """Test that on python 2.x if an OSError is raised by gzip.open
        operation doesn't stop.
        """
        self._do_test(OSError, mocker)

    @skip.PY3
    def test_config_ioerror(self, mocker):
        """Test that on python 2.x if an IOError is raised by gzip.open
        operation doesn't stop.
        """
        self._do_test(IOError, mocker)

    @skip.PY2
    def test_config_filenotfounderror(self, mocker):
        """Test that on python 3.x if an FileNotFound is raised by gzip.open
        operation doesn't stop.
        """
        self._do_test(FileNotFoundError, mocker)

    @skip.PY2
    def test_config_permissionerror(self, mocker):
        """Test that on python 3.x if an PermissionError is raised by gzip.open
        operation doesn't stop.
        """
        self._do_test(PermissionError, mocker)

    def test_not_timestamps(self, mocker):
        """If _last_message is populated but doesn't have a valid timestamp
        then an PiglitFatalException shoudl be raised.
        """
        mocker.patch('framework.dmesg.subprocess.check_output',
                     mocker.Mock(return_value=b'foo\nbar\n'))
        # This error will work for python 2.x and 3.x
        mocker.patch('framework.dmesg.gzip.open', side_effect=OSError)

        with pytest.raises(exceptions.PiglitFatalError):
            dmesg.LinuxDmesg()

    @skip.linux
    def test_partial_wrap(self):
        """dmesg.LinuxDmesg.update_dmesg: correctly handles partial wrap.

        Since dmesg is a ringbuffer it can roll over, and we need to ensure
        that we're handling that correctly.
        """
        result = results.TestResult()

        mock_out = mock.Mock(return_value=b'[1.0]This\n[2.0]is\n[3.0]dmesg')
        with mock.patch('framework.dmesg.subprocess.check_output', mock_out):
            test = dmesg.LinuxDmesg()

        mock_out.return_value = b'[3.0]dmesg\n[4.0]whoo!'
        with mock.patch('framework.dmesg.subprocess.check_output', mock_out):
            test.update_result(result)

        assert result.dmesg == '[4.0]whoo!'

    @skip.linux
    def test_complete_wrap(self):
        """dmesg.LinuxDmesg.update_dmesg: correctly handles complete wrap.

        Since dmesg is a ringbuffer (at least on Linux) it can roll over, and we
        need to ensure that we're handling that correctly.
        """
        result = results.TestResult()

        mock_out = mock.Mock(return_value=b'[1.0]This\n[2.0]is\n[3.0]dmesg')
        with mock.patch('framework.dmesg.subprocess.check_output', mock_out):
            test = dmesg.LinuxDmesg()

        mock_out.return_value = b'[4.0]whoo!\n[5.0]doggy'
        with mock.patch('framework.dmesg.subprocess.check_output', mock_out):
            test.update_result(result)

        assert result.dmesg == '[4.0]whoo!\n[5.0]doggy'


class TestLinuxDmesg(object):
    """Tests for LinuxDmesg methods."""

    def test_update_result_sets_result_attr(self):
        """When update_dmesg is called on a value it should set the dmesg
        attribute of the results.
        """
        result = results.TestResult(status.PASS)

        with mock.patch('framework.dmesg.subprocess.check_output',
                        mock.Mock(return_value=b'[1.0]this')):
            test = dmesg.LinuxDmesg()
        with mock.patch('framework.dmesg.subprocess.check_output',
                        mock.Mock(
                            return_value=b'[1.0]this\n[2.0]is\n[2.5]dmesg!\n')):
            test.update_result(result)

        assert result.dmesg == '[2.0]is\n[2.5]dmesg!'

    def test_update_result_no_change(self):
        """When update_result is called but no changes to dmesg have occured it
        should not set the dmesg attribute.
        """
        result = results.TestResult('pass')
        result.dmesg = mock.sentinel.dmesg

        with mock.patch('framework.dmesg.subprocess.check_output',
                        mock.Mock(return_value=b'[1.0]this')):
            test = dmesg.LinuxDmesg()
            test.update_result(result)

        assert result.dmesg is mock.sentinel.dmesg

    def test_repr(self):
        with mock.patch('framework.dmesg.subprocess.check_output',
                        mock.Mock(return_value=b'[1.0]this')):
            assert repr(dmesg.LinuxDmesg()) == 'LinuxDmesg()'


class TestDummyDmesg(object):
    """Tests for the DummyDmesg class."""
    _Namespace = collections.namedtuple('_Namespace', ['dmesg', 'result'])

    @pytest.fixture
    def testers(self):
        # The setter for result.TestResult checks types, rather than trying to
        # make the sentinel pass that test, just make a mock that mostly acts
        # like TestResult
        result = mock.Mock(spec=results.TestResult)
        result.dmesg = mock.sentinel.dmesg
        result.result = mock.sentinel.result
        return self._Namespace(dmesg.DummyDmesg(), result)

    def test_update_result(self, testers):
        """DummyDmesg.update_results shouldn't do anything."""
        testers.dmesg.update_result(testers.result)
        assert testers.result.dmesg is mock.sentinel.dmesg
        assert testers.result.result is mock.sentinel.result

    def test_repr(self):
        assert repr(dmesg.DummyDmesg()) == 'DummyDmesg()'


def _name_get_dmesg(value):
    """Function that names TestGetDmesg.test_get_dmesg."""
    if isinstance(value, bool):
        return 'real' if not value else 'dummy'
    elif isinstance(value, six.text_type):
        return value
    elif isinstance(value, dmesg.BaseDmesg):
        return repr(value)
    else:
        raise Exception('unreachable')


class TestGetDmesg(object):
    """Tests for get_dmesg factory."""

    @pytest.mark.parametrize(
        'platform,dummy,expected',
        [
            ('win32', False, dmesg.DummyDmesg),
            ('win32', True, dmesg.DummyDmesg),
            skip.linux(('linux', False, dmesg.DummyDmesg)),
            skip.linux(('linux', True, dmesg.LinuxDmesg)),
        ],
        ids=_name_get_dmesg)
    def test_get_dmesg(self, platform, dummy, expected, mocker):
        """Test that get_dmesg returns the expected dmesg type on variuos
        platforms with various configurations.
        """
        mocker.patch('framework.dmesg.sys.platform', platform)

        with mock.patch('framework.dmesg.subprocess.check_output',
                        mock.Mock(return_value=b'[1.0]foo')):
            actual = dmesg.get_dmesg(not_dummy=dummy)

        # We don't want a subclass, we want the *exact* class. This is a
        # unittest after all
        assert type(actual) == expected  # pylint: disable=unidiomatic-typecheck
